%CD1
% Numerical solution of 1-D convection-diffusion equation with Dirichlet
% boundary conditions:
%	dv/dx - (1/Pe)d^2v/dx^2 = 0,  v(0) = a,   v(1) = b
% Central scheme for convection, local grid refinement

% Copyright 2001 P. Wesseling
% This program and its subprograms may be freely used, modified and distributed
% under the GNU General Public License: http://www.gnu.org/copyleft/gpl.html

% Theory is given in Sect. 2.3 of
%	Computational Fluid Dynamics
%	Lecture Notes by P. Wesseling
%	Department of Applied Mathematical Analysis, ITS, TU Delft
% See http://ta.twi.tudelft.nl/nw/users/wesseling/

% This program generates Figs. 2.2 - 2.6 in the Lecture Notes

% Functions called: exact_solution

		% .......................Input..............................
a = 0.2;	% Left Dirichlet value
b = 1.0;	% Right Dirichlet value
pe = 400;	% Peclet number
scheme = 3;	% Enter 1,2 or something else for upwind, central scheme (2.20)
		% or central scheme (2.21)  
m1 = 6;		% Number of cells outside refinement zone
m2 = 6;		% Number of cells inside refinement zone
refinement = 1;	% Enter 1 for for local grid refinement or something else for
		% uniform grid 
		% .....................End of input......................

if refinement == 1, del = 6/pe;		% del = size of refinement zone
else,		    del = m2/(m1+m2);
end

% 	Definition of cell numbering

%                              |<---- del----->|
%     x=0       m1 cells       |    m2 cells  x=1
% grid |---o---|---o---|---o---|-o-|-o-|-o-|-o-|
%      1   1   2   2   3   3   4 4 5 5 6 6 7 7 8

x = zeros(m1+m2+1,1);		   % x contains coordinates of cell boundaries
j = 1:(m1+1);  	   x(j) = (j-1)*(1-del)/m1;
j = (m1+2):(m1+m2+1);  x(j) = x(m1+1) + (j-1-m1)*del/m2; 

dx = diff(x);			   % dx contains cell sizes: dx(j) = x(j+1)-x(j)
n = length(x)-1;     
y = (x(1:n) + x(2:n+1))/2;	   % y contains coordinates of cell centers
dy = [dx(1)/2; diff(y); dx(n)/2];

	% Coefficients in numerical flux are called beta:
	% F_{j+1/2} = beta0(j)v(j) + beta1(j+1)v(j+1), see equation (2.27)
	
betadiffusion = 1./(pe*dy);	% Contribution of diffusion term
				
if scheme == 1			% Upwind scheme

  beta0 = 1 + betadiffusion(2:n+1);
  beta1 =   - betadiffusion(1:n);
  gamma0 = a*(1+(2/pe)/dx(1));			% Boundary corrections for
  gamma1 = -b*(2/pe)/dx(n);			% right-hand side
    
elseif scheme == 2		% Central scheme (2.29)

  beta0 = 1/2 + betadiffusion(2:n+1);
  beta1 = 1/2 - betadiffusion(1:n);
  beta0(n) = betadiffusion(n+1);
  beta1(1) = - betadiffusion(1);
  gamma0 = (1+(2/pe)/dx(1))*a;			% Boundary corrections for
  gamma1 = (1-(2/pe)/dx(n))*b;			% right-hand side
    
else				% Central scheme (2.30)

  beta0 = 0.5*dx(2:n)./dy(2:n) +  betadiffusion(2:n);
  beta1 = 0.5*dx(1:n-1)./dy(2:n) -  betadiffusion(2:n);
  beta0 = [beta0; betadiffusion(n+1)];
  beta1 = [ - betadiffusion(1); beta1];  
  gamma0 = (1+(2/pe)/dx(1))*a;			% Boundary corrections for
  gamma1 = (1-(2/pe)/dx(n))*b;			% right-hand side
  
end
	
f = zeros(size(y));		% Right-hand side
f(1) = f(1) + gamma0; f(n) = f(n) - gamma1;

%	Numerical scheme gives system Av = f. 
% 	Diagonals of tridiagonal matrix A are stored in
%	[-beta0 beta0-beta1 beta1]:
% 	A(j,j-1) = -beta0(j-1);  
%	A(j,j) = beta0(j) - beta1(j);
%	A(j,j+1) = beta1(j+1);

A = spdiags([-beta0 beta0-beta1 beta1], -1:1, n, n);
numerical_solution = A\f;

% Computation of error norms
error = numerical_solution - exact_solution(y,pe,a,b); 
n = length(y); norm1 = norm(error,1)/n
norm2 = norm(error,2)/sqrt(n)
norminf = norm(error,inf)

% Make plot
clf, figure(1), hold on
plot(y,numerical_solution,'*','markersize',13)
fplot('exact_solution',[0 1],[],[],'-',pe,a,b)
h = findobj; set(h(3),'FontSize',18)
if scheme == 1,  	tt = 'Upwind scheme';
elseif scheme == 2,  	tt = 'Central scheme (2.20)';
else, 			tt = 'Central scheme (2.21)';
end
title(['Pe=',num2str(pe),',  ',num2str(n),' cells, ',tt],'fontsize',20)
box on
