%NS2
% Numerical solution of 2-D instationary Navier-Stokes equations.
% Rectangular domain; nonuniform Cartesian staggered grid.
% Time discretization with Adams-Bashforth-Crank-Nicolson scheme
% Pressure-correction method.
% Central or upwind scheme for inertia terms.

% Copyright 2001 P. Wesseling
% This program and its subprograms may be freely used, modified and distributed
% under the GNU General Public License: http://www.gnu.org/copyleft/gpl.html

% Theory is given in Chapter 6 of
% 	Principles of Computational Fluid Dynamics, by P. Wesseling
% 	Springer-Verlag, Berlin etc., 2001. ISBN 3-540-67853-0
% See http://dutita0.twi.tudelft.nl/nw/users/wesseling/

% Theory is also given in Chapter 5 of
%	Computational Fluid Dynamics
%	Lecture Notes by P. Wesseling
%	Department of Applied Mathematical Analysis, ITS, TU Delft
% See http://ta.twi.tudelft.nl/nw/users/wesseling/

% This program generates Fig.5.10 in the Lecture Notes.

% Functions called: startup,problem_specification, grid_generation, draw_grid,
%		    tau.m, viscous_matrix, grad_and_div, initial_condition, 
%		    right_hand_side, inertia_matrix, errornorm, isobarplot,
%		    streamlineplot, velocity_profile, relchangeplot  

startup, format compact, clear all
time0 = clock;
global geval n u0 v0 J K yu yv yseglen alpha
% ........................................Input..............................
geval = 1;	% Enter 1 for horizontal Poiseuille flow to the right
		%  or 2 for vertical Poiseuille flow
		%  or 3 for backward facing step
		%  or 4 for driven cavity
		%  or 5 for uniform flow under angle alpha without segmentation
		%  or 6 for uniform flow under angle alpha with segmentation
		%  or 7 for horizontal Poiseuille flow to the left
		
problem_specification	% Prepare file problem_specification.m as part of input
% User must prepare files ubd.m, vbd.m, pbd.m to specify boundary values 

% .................................End of input......................

grid_generation
draw_grid

dt = tau(Re, umax, vmax, hx, hy, central); % Stability time step restriction
	% for Adams-Bashforth-Crank-Nicolson according to Sect. 5.8 of
	% "Principles of Computational Fluid Dynamics" or Sect. ?? of
	% "Computational Fluid Dynamics" (Lecture Notes)
disp(['tau = ',num2str(dt)])
dt = 0.8*dt;
disp(['dt = ',num2str(dt)])

h = waitbar(0,'Computing...');
viscous_matrix		% Generate viscous matrices Bu, Bv and Masku, Maskv
grad_and_div		% Generate pressure gradient matrices Pu, Pv and 
			%  divergence matrices Du, Dv
initial_condition	% Generate initial u0, v0, p0 
tic		 
[Lp,Up] = lu([Du Dv]*[Pu;Pv]);	% LU decomposition of pressure correction matrix
tijd = toc; disp(['Pres. corr. matrix LU time = ',num2str(tijd)])
tic
[Lu,Uu] = lu(speye(length(u0)) + omega*dt*Bu);	 % LU decomposition of implicit
[Lv,Uv] = lu(speye(length(v0)) + omega*dt*Bv);	 %  part of operator
tijd = toc; disp(['Viscous matrices LU time = ',num2str(tijd)])
if max(abs(sum(([Du Dv]*[Pu;Pv])'))) < 10*eps		
  % Do consistency check
end

nstep = floor(tend/dt);	% Number of time steps
%nstep = 5;
relchange = zeros(nstep,1);	% For plotting relative change per time step

t = 0; n = 0; 
right_hand_side		% Generate right-hand sides ru, rv
u1 = u0; v1 = v0;
inertia_term
Bu = Masku - dt*(1-omega)*Bu; Bv = Maskv - dt*(1-omega)*Bv;

for n = 1:nstep		% Time-stepping with Adams-Bashforth-omega scheme 
  t = n*dt;
  u0 = u1; v0 = v1;
%  right_hand_side	% Generate right-hand sides ru, rv; can be skipped
			% if boundary conditions not time-dependent
  
  % Navier-Stokes, predictor:
  cu00 = cu0; cv00 = cv0; 
  inertia_term 
  if n == 1, tic, end
%  rum = Bu*u0 + dt*(ru - Pu*p0 - 1.5*cu0 + cu00/2);
%  rvm = Bv*v0 + dt*(rv - Pv*p0 - 1.5*cv0 + cv00/2);
%  u1 = Uu\(Lu\rum); v1 = Uv\(Lv\rvm);
  u1 = Uu\(Lu\(Bu*u0 + dt*(ru - Pu*p0 - 1.5*cu0 + cu00/2)));
  v1 = Uv\(Lv\(Bv*v0 + dt*(rv - Pv*p0 - 1.5*cv0 + cv00/2)));
  if n == 1, tijd = toc; disp(['velocity solve time = ',num2str(tijd)]), tic,end
  % End of Navier-Stokes predictor
   
  dp = Up\(Lp\(Du*u1 + Dv*v1));	p0 = p0 + dp/dt;
  u1 = u1 - Pu*dp;	v1 = v1 - Pv*dp;
  if n == 1, tijd = toc; disp(['press. corr. time = ',num2str(tijd)]),end
  mm = max([norm(u1,inf),norm(v1,inf)]); relchange(n) = max([norm(u1-u0,inf)...
  /mm,norm(v1-v0,inf)/mm, norm(dp,inf)/(norm(p0,inf) + 0.5*mm^2)])/dt;
  waitbar(n/nstep)
end

disp(['Relative change = ',num2str(relchange(end))])
divergence = norm(Du*u1 + Dv*v1,inf);
if (geval==1)|(geval==2)
  errornorm		% Maximum norm of error in outflow profile if exact
			% solution is available
end
close(h)
tic
isobarplot		% Plot of isobars and velocity vectors
streamlineplot 		% Plot of streamlines and velocity vectors
velocity_profile	% Plot of velocity profiles
relchangeplot		% Plot of relative change per time step
tijd = toc; disp(['Plotting time = ',num2str(tijd)])
disp(['Total time = ',num2str(etime(clock,time0))])
