%NS3
% Numerical solution of 2-D stationary Navier-Stokes equations.
% Rectangular domain; nonuniform Cartesian staggered grid.
% Direct solution.
% Picard linearization.
% Central or upwind scheme for inertia terms.

% Copyright 2001 P. Wesseling
% This program and its subprograms may be freely used, modified and distributed
% under the GNU General Public License: http://www.gnu.org/copyleft/gpl.html

% Theory is given in Chapter 6 of
% 	Principles of Computational Fluid Dynamics, by P. Wesseling
% 	Springer-Verlag, Berlin etc., 2001. ISBN 3-540-67853-0
% See http://dutita0.twi.tudelft.nl/nw/users/wesseling/

% Theory is also given in Chapter 6 of
%	Computational Fluid Dynamics
%	Lecture Notes by P. Wesseling
%	Department of Applied Mathematical Analysis, ITS, TU Delft
% See http://ta.twi.tudelft.nl/nw/users/wesseling/

% This program generates Figs. 6.2, 6.3 and 6.5 in the Lecture Notes.

% Functions called: startup, problem_specification, grid_generation, draw_grid,
%		    viscous_matrix, grad_and_div, initial_condition, 
%		    right_hand_side, inertia_matrix, errornorm, isobarplot,
%		    streamlineplot, velocity_profile, relchangeplot  

startup, format compact, clear all
time0 = clock;
global geval n u0 v0 J K yu yv yseglen alpha

% ........................................Input..............................
geval = 1;	% Enter 1 for horizontal Poiseuille flow to the right
		%  or 2 for vertical Poiseuille flow
		%  or 3 for backward facing step
		%  or 4 for driven cavity
		%  or 5 for uniform flow under angle alpha without segmentation
		%  or 6 for uniform flow under angle alpha with segmentation
		%  or 7 for horizontal Poiseuille flow to the left
		%  or 8 for uniform horizontal flow with outflow bound. cond.
		%	at three sides
		%  or 9 for uniform horizontal flow with inflow bound. cond.
		%	at three sides
		
problem_specification	% Prepare file problem_specification.m as part of input

% User must also prepare files ubd.m, vbd.m, pbd.m to specify boundary values. 
% Choice between Picard linearization and extrapolated Picard linearization is
% made at the beginning of inertia_matrix.m

nstep = 90;	% Maximum number of Picard iterations.
%nstep = 2;
% .................................End of input......................

grid_generation
draw_grid
h = waitbar(0,'Computing...');
viscous_matrix		% Generate viscous matrices Bu, Bv and Masku, Maskv
  Masku = speye(size(Masku))-Masku;	% Used to put ones in matrices for
  Maskv = speye(size(Maskv))-Maskv;	% application Dirichlet bound. cond.
grad_and_div		% Generate pressure gradient matrices Pu, Pv and 
initial_condition	% Generate initial u0, v0, p0 
			%  divergence matrices Du, Dv

if max(abs(sum(([Du Dv]*[Pu;Pv])'))) < 10*eps		
  % Do consistency check
end

relchange = zeros(nstep,1);	% For plotting relative change per time step
n = 0; t = 0; dt =1; 		% Required for correct right-hand side
right_hand_side		% Generate right-hand sides ru, rv and boundary
			% values for velocity and pressure
u1 = u0; v1 = v0; p1 = p0;
Re_input = Re; %##### We will go to high Re by continuation
for n = 1:nstep		% Picard iteration
  
  if nstep<30 & n<30, Re = min([3000,Re_input]); end
  if nstep<20 & n<20, Re = min([2000,Re_input]); end
  if nstep<10 & n<10, Re = min([1000,Re_input]); end
  if n>29, Re = Re_input; end
  u0 = u1; v0 = v1; p0 = p1;

  inertia_matrix	% Generate inertia matrices and adapt right-hand sides 
  if n == 1|n == 2, tic, end
  Cu = Bu + Cu + Masku; Cv = Bv + Cv + Maskv;
  A = [Cu  zeros(size(Cu,1),size(Cv,2))  Pu
       zeros(size(Cv,1),size(Cu,2))  Cv  Pv
       Du  Dv  zeros(size(Du,1),size(Pu,2))];
  
%  A*[u0;v0;p0]-[ru+rum; rv+rvm; zeros(size(Du,1),1)]
  
  uu = A\[ru+rum; rv+rvm; zeros(size(Du,1),1)];
  u1 = uu(1:size(XU(:))); v1 = uu(1+size(XU(:)): size(XU(:))+size(XV(:)));
  p1 = uu(1+size(XU(:))+size(XV(:)):end);
   	
  if n == 1|n == 2, tijd = toc; disp(['solution time = ',num2str(tijd)]), end
  
  mm = max([norm(u1,inf),norm(v1,inf)]);
 if geval == 4 			% Pressure is undetermined up to a constant 
   relchange(n) = max([norm(u1-u0,inf)/mm,norm(v1-v0,inf)/mm,])/dt;
 else
    relchange(n) = max([norm(u1-u0,inf)...
    /mm,norm(v1-v0,inf)/mm, norm(p1-p0,inf)/(norm(p0,inf) + 0.5*mm^2)])/dt;
  end    
  waitbar(n/nstep)
end

disp(['Relative change = ',num2str(relchange(end))])
divergence = norm(Du*u1 + Dv*v1,inf);
if (geval==1)|(geval==2)
  errornorm		% Maximum norm of error in outflow profile if exact
			% solution is available
end
close(h)
tic
isobarplot		% Plot of isobars and velocity vectors
streamlineplot 		% Plot of streamlines and velocity vectors
velocity_profile	% Plot of velocity profiles
relchangeplot		% Plot of relative change per time step
tijd = toc; disp(['Plotting time = ',num2str(tijd)])
disp(['Total time = ',num2str(etime(clock,time0))])
